/**
 * Makes http calls to remote resources as `Cmd`s.
 *
 * See [Http](https://package.elm-lang.org/packages/elm/http/latest/Http) Elm package.
 *
 * @since 0.5.0
 */
import * as E from 'fp-ts/es6/Either';
import * as O from 'fp-ts/es6/Option';
import * as TE from 'fp-ts/es6/TaskEither';
import { Cmd } from './Cmd';
import { Decoder } from './Decode';
import Option = O.Option;
import Either = E.Either;
import TaskEither = TE.TaskEither;
/**
 * @category model
 * @since 0.5.0
 */
export declare type Method = 'GET' | 'POST' | 'PUT' | 'PATCH' | 'DELETE';
/**
 * @category model
 * @since 0.5.0
 */
export interface Request<A> {
    expect: Decoder<A>;
    url: string;
    method: Method;
    headers: Record<string, string>;
    body?: unknown;
    timeout: Option<number>;
    withCredentials: boolean;
}
/**
 * @category model
 * @since 0.5.0
 */
export interface Response<Body> {
    url: string;
    status: {
        code: number;
        message: string;
    };
    headers: Record<string, string>;
    body: Body;
}
/**
 * @category model
 * @since 0.5.0
 */
export declare type HttpError = {
    readonly _tag: 'BadUrl';
    readonly value: string;
} | {
    readonly _tag: 'Timeout';
} | {
    readonly _tag: 'NetworkError';
    readonly value: string;
} | {
    readonly _tag: 'BadStatus';
    readonly response: Response<string>;
} | {
    readonly _tag: 'BadPayload';
    readonly value: string;
    readonly response: Response<string>;
};
/**
 * @category destructors
 * @since 0.5.0
 */
export declare function toTask<A>(req: Request<A>): TaskEither<HttpError, A>;
/**
 * Executes as `Cmd` the provided call to remote resource, mapping result to a `Msg`.
 * @category utils
 * @since 0.5.0
 */
export declare function send<A, Msg>(f: (e: Either<HttpError, A>) => Msg): (req: Request<A>) => Cmd<Msg>;
/**
 * @category creators
 * @since 0.5.0
 */
export declare function get<A>(url: string, decoder: Decoder<A>): Request<A>;
/**
 * @category creators
 * @since 0.5.0
 */
export declare function post<A>(url: string, body: unknown, decoder: Decoder<A>): Request<A>;
